Option Explicit

Const ForReading      = 1
Const ForWriting      = 2
Const ForAppending    = 8
Const BinaryCompare   = 0
Const TextCompare     = 1
Const DatabaseCompare = 2

Dim arrCP(255)
Dim blnBreak, blnCodePage, blnIndent, blnOverWrite, blnTabConvert, blnUnicode
Dim i, intCodePage, intIndent, intTabConvert, intValidArgs
Dim objFSO, objInFile, objOutFile, objStreamIn, wshShell
Dim strCommand, strInFile, strOutFile, strScriptEnv, strTab, strText

' Check command line arguments
With WScript.Arguments
	' Check for /?
	If .Named.Exists( "?" ) Then
		Syntax ""
	End If
	' Initialize counter for valid named arguments
	intValidArgs = 0
	' /BR = add "<BR>" at the end of each line
	If .Named.Exists( "BR" ) Then
		blnBreak = True
		intValidArgs  = intValidArgs + 1
	Else
		blnIndent = False
	End If
	' /C:nnn = use codepage nnn for character translation
	If .Named.Exists( "C" ) Then
		intValidArgs  = intValidArgs + 1
		blnCodePage   = True
		If .Named( "C" ) = "" Then
			intCodePage = GetCodePage( )
			If intCodePage = 0 Then
				Syntax "ERROR: Unable to determine current codepage"
			Else
			Select Case intCodePage
				Case "437"
					FillArrayCP437
				Case "850"
					FillArrayCP850
				Case "858"
					FillArrayCP858
				Case "1252"
					FillArrayCP1252
				Case Else
					Syntax "ERROR: Invalid codepage specified"
			End Select
			End If
		Else
			intCodePage = .Named( "C" )
			Select Case intCodePage
				Case "437"
					FillArrayCP437
				Case "850"
					FillArrayCP850
				Case "858"
					FillArrayCP858
				Case "1252"
					FillArrayCP1252
				Case Else
					Syntax "ERROR: Invalid codepage specified"
			End Select
		End If
	Else
		blnCodePage = False
		FillArrayCP1252
	End If
	' /I:n = indent each line with n spaces (default 8)
	If .Named.Exists( "I" ) Then
		blnIndent = True
		intIndent = CInt( .Named( "I" ) )
		If intIndent = 0 Then intIndent = 8
		intValidArgs  = intValidArgs + 1
	Else
		blnIndent = False
	End If
	' /T:n = replace tabs with n times "&nbsp;" (default 4),
	'        and also replace double spaces with double "&nbsp;"
	If .Named.Exists( "T" ) Then
		blnTabConvert = True
		intTabConvert = CInt( .Named( "T" ) )
		If intTabConvert = 0 Then intTabConvert = 4
		intValidArgs  = intValidArgs + 1
	Else
		blnTabConvert = False
	End If
	' /U = treat text as Unicode (default is ASCII)
	If .Named.Exists( "U" ) Then
		blnUnicode   = True
		intValidArgs = intValidArgs + 1
	Else
		blnUnicode = False
	End If
	' /Y = overwrite existing target file
	If .Named.Exists( "Y" ) Then
		blnOverWrite = True
		intValidArgs = intValidArgs + 1
	Else
		blnOverWrite = False
	End If
	' Check if any "undefined" switches were used
	If intValidArgs <> .Named.Count Then Syntax "ERROR: Invalid switches"

	' Check for forbidden combination of switches
	If blnCodePage And blnUnicode Then Syntax "ERROR: Switches /C and /U are mutually exclusive"

	' Check the "unnamed" arguments (source and optionally target file)
	Select Case .Unnamed.Count
		Case 1
			strInFile  = .Unnamed( 0 )
		Case 2
			strInFile  = .Unnamed( 0 )
			strOutFile = .Unnamed( 1 )
		Case Else
			Syntax "ERROR: Invalid argument or invalid number of arguments"
	End Select
End With

' Create a file system object
Set objFSO = CreateObject( "Scripting.FileSystemObject" )

With objFSO
	' Abort if the source file doesn't exist
	If Not .FileExists( strInFile ) Then
		Syntax "ERROR: File not found: " & strInFile
	End If
	' If no target file was specified, we will use the file
	' PATH and NAME of the source file, with a .SRC extension
	If WScript.Arguments.Unnamed.Count = 1 Then
		strOutFile = .BuildPath( .GetParentFolderName( strInFile ), _
		                         .GetBaseName( strInFile ) ) & ".src"
	End If
	' Abort if the target file exists, unless Overwrite was specified
	If .FileExists( strOutFile ) And Not blnOverWrite Then
		Syntax "ERROR: File exists: " & strOutFile
	End If
	' Open the source file for reading and the target file for writing
	Set objInFile   = .GetFile( strInFile )
	Set objStreamIn = objInFile.OpenAsTextStream( ForReading, blnUnicode )
	Set objOutFile  = .OpenTextFile( strOutFile, ForWriting, True, blnUnicode )
End With

' Read the text from the source file and close it
strText = objStreamIn.ReadAll( )
objStreamIn.Close

' Replace each "special" character in the text by its HTML entity
strText = Replace( strText, "&",  "&amp;",    1, -1, vbTextCompare )
strText = Replace( strText, "<",  "&lt;",     1, -1, vbTextCompare )
strText = Replace( strText, ">",  "&gt;",     1, -1, vbTextCompare )
strText = Replace( strText, "^",  "&circ;",   1, -1, vbTextCompare )
For i = 128 To 255
	If arrCP(i) <> "" Then
		strText = Replace( strText, Chr(i), arrCP(i), 1, -1, vbTextCompare )
	End If
Next

' Replace tabs if specified on the command line
If blnTabConvert Then
	For i = 1 To intTabConvert
		strTab = strTab & "&nbsp;"
	Next
	strText = Replace( strText, vbTab, strTab, 1, -1, vbTextCompare )
	strTab = "&nbsp;&nbsp;"
	strText = Replace( strText, "  ",  strTab, 1, -1, vbTextCompare )
End If
' Add indents if specified on the command line
If blnIndent Then
	strText = Replace( strText, vbCrLf, vbCrLf & Space( intIndent ), 1, -1, vbTextCompare )
	strText = Space( intIndent ) & strText
End If
' Add <BR> if specified on the command line
If blnBreak Then
	strText = Replace( strText, vbCrLf, _
	          "<br>" & vbCrLf, 1, -1, vbTextCompare )
End If

' Write the modified text to the target file and close the file
objOutFile.Write( strText )
objOutFile.Close

' Release the objects
Set objInFile = Nothing
Set objOutFile = Nothing
Set objFSO = Nothing


Sub Syntax( myMsg )
	If myMsg <> "" Then myMsg = myMsg & vbCrLf & vbCrLf
	myMsg = myMsg _
	      & "Txt2Src.vbs,  Version 2.00" & vbCrLf _
	      & "Convert special characters in a text file to HTML entities" _
	      & vbCrLf & vbCrLf _
	      & "Usage:  TXT2SRC  source  [target]  [/BR]  [/C[:n]]  [/I:n]  [/T:n]  [/U]  [/Y]" _
	      & vbCrLf & vbCrLf _
	      & "Where:  ""source""   is the text file to be read" _
	      & vbCrLf _
	      & "        ""target""   is the file which will contain the converted text" _
	      & vbCrLf _
	      & "                   (default is source path and name, with "".src"" extension)" _
	      & vbCrLf _
	      & "        ""/BR""      add ""<br>"" at each line break" _
	      & vbCrLf _
	      & "        ""/C[:n]""   use alternative codepage instead of 1252; valid n=437," _
	      & vbCrLf _
	      & "                   n=850, n=858 or n=1252, default n=current (" & GetCodePage( ) & ")" _
	      & vbCrLf _
	      & "        ""/I[:n]""   indent each line with n spaces (default n=8)" _
	      & vbCrLf _
	      & "        ""/T[:n]""   convert tabs to n non-breaking spaces (default n=4)" _
	      & vbCrLf _
	      & "                   and double spaces to double non-breaking spaces" _
	      & vbCrLf _
	      & "        ""/U""       treat text as Unicode (default ASCII)" _
	      & vbCrLf _
	      & "        ""/Y""       overwrite the target file if it exists" _
	      & vbCrLf & vbCrLf _
	      & "Note:   Switches /C and /U are mutually exclusive." _
	      & vbCrLf & vbCrLf _
	      & "Written by Rob van der Woude" & vbCrLf _
	      & "http://www.robvanderwoude.com"
	WScript.Echo myMsg
	WScript.Quit 1
End Sub


Function GetCodePage( )
	Dim arrCodepage, objExec, wshShell
	GetCodePage = 0
	Set wshShell = CreateObject( "WScript.Shell" )
	Set objExec  = wshShell.Exec( "%ComSpec% /C CHCP" )
	Do Until objExec.Status
		Wscript.Sleep 100
	Loop
	If objExec.StdErr.ReadAll = "" Then
		arrCodepage = Split( objExec.StdOut.ReadAll, " " )
		GetCodePage = CInt( arrCodepage( UBound( arrCodepage ) ) )
	End If
	Set objExec  = Nothing
	Set wshShell = Nothing
End Function


Sub FillArrayCP437
	Dim i
	For i = 0 To 255
		arrCP(i) = ""
	Next
	arrCP(7)   = "<em>{BEL}</em>"
	arrCP(8)   = "<em>{Backspace}</em>"
	arrCP(9)   = vbTab
	arrCP(26)  = "<em>{EOF}</em>"
	arrCP(27)  = "<em>{ESC}</em>"
	arrCP(128) = "&Ccedil;"
	arrCP(129) = "&uuml;"
	arrCP(130) = "&eacute;"
	arrCP(131) = "&acirc;"
	arrCP(132) = "&auml;"
	arrCP(133) = "&agrave;"
	arrCP(134) = "&aring;"
	arrCP(135) = "&ccedil;"
	arrCP(136) = "&ecirc;"
	arrCP(137) = "&euml;"
	arrCP(138) = "&egrave;"
	arrCP(139) = "&iuml;"
	arrCP(140) = "&icirc;"
	arrCP(141) = "&igrave;"
	arrCP(142) = "&Auml;"
	arrCP(143) = "&Aring;"
	arrCP(144) = "&Eacute;"
	arrCP(145) = "&aelig;"
	arrCP(146) = "&AElig;"
	arrCP(147) = "&ocirc;"
	arrCP(148) = "&ouml;"
	arrCP(149) = "&ograve;"
	arrCP(150) = "&ucirc;"
	arrCP(151) = "&ugrave;"
	arrCP(152) = "&yuml;"
	arrCP(153) = "&Ouml;"
	arrCP(154) = "&Uuml;"
	arrCP(155) = "&cent;"
	arrCP(156) = "&pound;"
	arrCP(157) = "&yen;"
	arrCP(158) = "<em>{Pts}</em>"
	arrCP(159) = "&fnof;"
	arrCP(160) = "&aacute;"
	arrCP(161) = "&iacute;"
	arrCP(162) = "&oacute;"
	arrCP(163) = "&uacute;"
	arrCP(164) = "&ntilde;"
	arrCP(165) = "&Ntilde;"
	arrCP(166) = "&ordf;"
	arrCP(167) = "&ordm;"
	arrCP(168) = "&iquest;"
	arrCP(170) = "&not;"
	arrCP(171) = "&frac12;"
	arrCP(172) = "&frac14;"
	arrCP(173) = "&iexcl;"
	arrCP(174) = "&laquo;"
	arrCP(175) = "&raquo;"
	arrCP(224) = "&alpha;"
	arrCP(225) = "&szlig;"
	arrCP(226) = "&gamma;"
	arrCP(227) = "&pi;"
	arrCP(228) = "&Sigma;"
	arrCP(229) = "&sigma;"
	arrCP(230) = "&micro;"
	arrCP(231) = "&tau;"
	arrCP(232) = "&phi;"
	arrCP(233) = "&Theta;"
	arrCP(234) = "&Omega;"
	arrCP(235) = "&delta;"
	arrCP(236) = "&infin;"
	arrCP(237) = "&Phi;"
	arrCP(238) = "&epsilon;"
	arrCP(239) = "&cap;"
	arrCP(240) = "&equiv;"
	arrCP(241) = "&plusmn;"
	arrCP(242) = "&ge;"
	arrCP(243) = "&le;"
	arrCP(246) = "&divide;"
	arrCP(247) = "&asymp;"
	arrCP(248) = "&deg;"
	arrCP(250) = "&middot;"
	arrCP(251) = "&radic;"
	arrCP(252) = "<sup>n</sup>"
	arrCP(253) = "&sup2;"
	arrCP(255) = "&nbsp;"
End Sub


Sub FillArrayCP850
	FillArrayCP437
	arrCP(155) = "&oslash;"
	arrCP(157) = "&Oslash;"
	arrCP(158) = "&times;"
	arrCP(169) = "&reg;"
	arrCP(181) = "&Aacute;"
	arrCP(182) = "&Acirc;"
	arrCP(183) = "&Agrave;"
	arrCP(181) = "&copy;"
	arrCP(198) = "&atilde;"
	arrCP(199) = "&Atilde;"
	arrCP(207) = "&curren;"
	arrCP(208) = "&eth;"
	arrCP(209) = "&ETH;"
	arrCP(210) = "&Ecirc;"
	arrCP(211) = "&Euml;"
	arrCP(212) = "&Egrave;"
	arrCP(213) = "&#305;"
	arrCP(214) = "&Iacute;"
	arrCP(215) = "&Icirc;"
	arrCP(216) = "&Iuml;"
	arrCP(221) = "&brvbar;"
	arrCP(222) = "&Igrave;"
	arrCP(224) = "&Oacute;"
	arrCP(226) = "&Ocirc;"
	arrCP(227) = "&Ograve;"
	arrCP(228) = "&otilde;"
	arrCP(229) = "&Otilde;"
	arrCP(231) = "&thorn;"
	arrCP(232) = "&THORN;"
	arrCP(233) = "&Uacute;"
	arrCP(234) = "&Ucirc;"
	arrCP(235) = "&Ugrave;"
	arrCP(236) = "&yacute;"
	arrCP(237) = "&Yacute;"
	arrCP(238) = "&#175;"
	arrCP(239) = "&acute;"
	arrCP(240) = "&shy;'"
	arrCP(241) = "&plusmn;"
	arrCP(242) = "&#8215;"
	arrCP(243) = "&frac34;"
	arrCP(244) = "&para;"
	arrCP(245) = "&sect;"
	arrCP(247) = "&cedil;"
	arrCP(249) = "&uml;"
	arrCP(251) = "&sup1;"
	arrCP(252) = "&sup3;"
	arrCP(255) = "&nbsp;"
End Sub


Sub FillArrayCP858
	FillArrayCP850
	arrCP(213) = "&euro;"
End Sub


Sub FillArrayCP1252
	FillArrayCP437
	arrCP(128) = "&euro;"
	arrCP(129) = ""
	arrCP(130) = "&sbquo;"
	arrCP(131) = "&fnof;"
	arrCP(132) = "&bdquo;"
	arrCP(133) = "&hellip;"
	arrCP(134) = "&dagger;"
	arrCP(135) = "&Dagger;"
	arrCP(136) = "&circ;"
	arrCP(137) = "&permil;"
	arrCP(138) = "&Scaron;"
	arrCP(139) = "&lsaquo;"
	arrCP(140) = "&OElig;"
	arrCP(141) = ""
	arrCP(142) = "&#381;"
	arrCP(143) = ""
	arrCP(144) = ""
	arrCP(145) = "&lsquo;"
	arrCP(146) = "&rsquo;"
	arrCP(147) = "&ldquo;"
	arrCP(148) = "&rdquo;"
	arrCP(149) = "&bull;"
	arrCP(150) = "&ndash;"
	arrCP(151) = "&mdash;"
	'arrCP(152) = "&tilde;"
	arrCP(153) = "&trade;"
	arrCP(154) = "&scaron;"
	arrCP(155) = "&rsaquo;"
	arrCP(156) = "&oelig;"
	arrCP(157) = ""
	arrCP(158) = "&#382;"
	arrCP(159) = "&Yuml;"
	arrCP(160) = "&nbsp;"
	arrCP(161) = "&iexcl;"
	arrCP(162) = "&cent;"
	arrCP(163) = "&pound;"
	arrCP(164) = "&curren;"
	arrCP(165) = "&yen;"
	arrCP(166) = "&brvbar;"
	arrCP(167) = "&sect;"
	arrCP(168) = "&uml;"
	arrCP(169) = "&copy;"
	arrCP(170) = "&ordf;"
	arrCP(171) = "&laquo;"
	arrCP(172) = "&not;"
	arrCP(173) = "&shy;"
	arrCP(174) = "&reg;"
	arrCP(175) = "&macron;"
	arrCP(176) = "&deg;"
	arrCP(177) = "&plusmn;"
	arrCP(178) = "&sup2;"
	arrCP(179) = "&sup3;"
	arrCP(180) = "&acute;"
	arrCP(181) = "&micro;"
	arrCP(182) = "&para;"
	arrCP(183) = "&middot;"
	arrCP(184) = "&cedil;"
	arrCP(185) = "&sup1;"
	arrCP(186) = "&ordm;"
	arrCP(187) = "&raquo;"
	arrCP(188) = "&frac14;"
	arrCP(189) = "&frac12;"
	arrCP(190) = "&frac34;"
	arrCP(191) = "&iquest;"
	arrCP(192) = "&Agrave;"
	arrCP(193) = "&Aacute;"
	arrCP(194) = "&Acirc;"
	arrCP(195) = "&Atilde;"
	arrCP(196) = "&Auml;"
	arrCP(197) = "&Aring;"
	arrCP(198) = "&AElig;"
	arrCP(199) = "&Ccedil;"
	arrCP(200) = "&Egrave;"
	arrCP(201) = "&Eacute;"
	arrCP(202) = "&Ecirc;"
	arrCP(203) = "&Euml;"
	arrCP(204) = "&Igrave;"
	arrCP(205) = "&Iacute;"
	arrCP(206) = "&Icirc;"
	arrCP(207) = "&Iuml;"
	arrCP(208) = "&ETH;"
	arrCP(209) = "&Ntilde;"
	arrCP(210) = "&Ograve;"
	arrCP(211) = "&Oacute;"
	arrCP(212) = "&Ocirc;"
	arrCP(213) = "&Otilde;"
	arrCP(214) = "&Ouml;"
	arrCP(215) = "&times;"
	arrCP(216) = "&Oslash;"
	arrCP(217) = "&Ugrave;"
	arrCP(218) = "&Uacute;"
	arrCP(219) = "&Ucirc;"
	arrCP(220) = "&Uuml;"
	arrCP(221) = "&Yacute;"
	arrCP(222) = "&THORN;"
	arrCP(223) = "&szlig;"
	arrCP(224) = "&agrave;"
	arrCP(225) = "&aacute;"
	arrCP(226) = "&acirc;"
	arrCP(227) = "&atilde;"
	arrCP(228) = "&auml;"
	arrCP(229) = "&aring;"
	arrCP(230) = "&aelig;"
	arrCP(231) = "&ccedil;"
	arrCP(232) = "&egrave;"
	arrCP(233) = "&eacute;"
	arrCP(234) = "&ecirc;"
	arrCP(235) = "&euml;"
	arrCP(236) = "&igrave;"
	arrCP(237) = "&iacute;"
	arrCP(238) = "&icirc;"
	arrCP(239) = "&iuml;"
	arrCP(240) = "&eth;"
	arrCP(241) = "&ntilde;"
	arrCP(242) = "&ograve;"
	arrCP(243) = "&oacute;"
	arrCP(244) = "&ocirc;"
	arrCP(245) = "&otilde;"
	arrCP(246) = "&ouml;"
	arrCP(247) = "&divide;"
	arrCP(248) = "&oslash;"
	arrCP(249) = "&ugrave;"
	arrCP(250) = "&uacute;"
	arrCP(251) = "&ucirc;"
	arrCP(252) = "&uuml;"
	arrCP(253) = "&yacute;"
	arrCP(254) = "&thorn;"
	arrCP(255) = "&yuml;"
End Sub
